import sys, exceptions, os, optparse, math
from twisted.python import log
from twisted.internet import reactor
from pyGsi import proxylib
from pyGridWare.common.rftlib import RFT

class MissingServiceURL(exceptions.Exception): pass

class OptionHolder: pass

from twisted.cred import portal,checkers
from twisted.conch import manhole, manhole_ssh
from twisted.application import internet

def ManholeFactory(namespace,**passwords) :
    realm = manhole_ssh.TerminalRealm()
    def getManhole(_): return manhole.Manhole(namespace)
    realm.chainedProtocolFactory.protocolFactory = getManhole
    p = portal.Portal(realm)
    p.registerChecker(checkers.InMemoryUsernamePasswordDatabaseDontUse(**passwords))
    f = manhole_ssh.ConchFactory(p)
    return f


def setupManhole(rft):
    f = ManholeFactory(locals(),
                       admin='acct')

    reactor.listenTCP(6023, f)

def main():
    ## Simple set of default options.
    options = OptionHolder()
    options.authz= None
    options.m2secureMessage= False
    options.m2crypto= True
    options.m2secureConv= None
    ## TODO: use pyGSI convenience function to retrieve this
    options.cert= '/tmp/x509up_u' + str(os.getuid())
    options.key= None
    options.wsAddressURI= 'http://schemas.xmlsoap.org/ws/2004/03/addressing'


    ## The RFT tool doesn't use the same command line argument as the
    ## options parser provided by pyGridWare.
    
    op = optparse.OptionParser()
    op.add_option("-u", "--url", action="store", dest="url",
        help="location of service", default=None)
    op.add_option("-d", "--debug",
        action="store", dest="debug", default=0,
        help="Debug Messages to file, if 1 send to stdout, if 0 then off.")


    (cloptions, args) = op.parse_args()
    if len(args)/2. != math.floor(len(args)/2.):
        print "Number of arguments must be even"
        return -1
    transferPairs = zip(args[::2], args[1::2])
    
    if cloptions.debug:
        log.startLogging(sys.stdout)

    if not cloptions.url:
        raise MissingServiceURL
    options.url=cloptions.url

    ## Certificate validity check (would do this later but call to GetKWPortArgs
    ## raises a generic RuntimeError)
    proxyfilename = proxylib.get_proxy_filename()
    if not os.path.exists(proxyfilename):
        print "Proxy file does not exist"
        return -1
    cert = proxylib.get_usercert(proxyfilename)
    if not proxylib.check_valid(cert):
        print "Cannot delegate an expired credential."
        sys.exit(-1)

    r = RFT(options, transferPairs)
    reactor.callWhenRunning(r.Transfer)
#    reactor.callWhenRunning(r.DelegateCredential)
    setupManhole(r)
    reactor.run()

if __name__ == '__main__':
    main()
