############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from zope.interface import Interface, Attribute


class IWSResource(Interface):
    """Represents a WSRF Resource, implies there will be
    a resource home indexed via addressing abstraction
    thru the resource home manager.
    """
    Attribute('home', 'resource home (manager)')
    
    
class IResourceHome(Interface):
    """Interface for ResourceHome, any class implementing this interface
    is expected to handle lifetime management of resources added.
    """
    def add(self, ctx, key=None):
        """save resource context referenced via key.
        """
    def create(self):
        """create a new key.
        """
    def delete(self, key):
        """immediately delete resource context referenced via key.
        """
    def get(self, key):
        """get resource context via key.
        """
    def setTerminationTime(self, key, terminationTime):
        """set terminationTime of resource, and schedule deletion of 
        resource context referenced via key.
        """
    def setAvatarId(self, avatarId):
        """set avatarId
        """
    def getAvatarId(self):
        """get avatarId
        """
    def getResources(self):
        """return all resource contexts.
        """
    def getResourceContext(self, ps, address):
        """return resource context.
        """

class IResourceHomeManager(IResourceHome):
    """ResourceHomeManager Interface
    """
    def newResourceHome(self):
        """return new resource home instance.
        """
    def setResourceHome(self, key, home):
        """add resource home to registry.
        """      
    def getResourceHome(self, key):
        """get resource home from registry using key.
        """

class IPersistent(Interface):
    """Interface for Persistent ResourceHome, any class implementing 
    this interface is expected to save all resource.
    """
    def save(self, key=None):
        """If key is None save all resources, else save only 
        resource specified via key.
        """
    def load(self):
        """Implement this method to load up state upon initialization.
        """



