############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import os, socket
from ConfigParser import ConfigParser
from twisted.application import service, internet
from twisted.web.server import Site
from twisted.web.resource import Resource, IResource
from twisted.web import static, script
from ZSI.address import Address
from ZSI.wstools.logging import setBasicLoggerDEBUG
setBasicLoggerDEBUG()
from pyGridWare.wsrf.notification.SubscriptionManager import SubscriptionManager
from pyGridWare.services.SubscriptionManager import SubscriptionManager as SMServiceBinding
from pyGridWare.utility.http import ChunkedHTTPChannel

# SET UP
import pyGridWare.utility.m2crypto
from pyGridWare.security.authz.resource import ResourcePortal

from twisted.internet import defer
from twisted.internet.defer import maybeDeferred
from twisted.python import failure, reflect, components
from twisted.cred import error
from zope import interface

from twisted.cred.portal import Portal, IRealm
from twisted.web.http import HTTPChannel
from twisted.python import failure, log
from twisted.web import server, http as _http
import string


# SSL SETUP 
import twisted.protocols.policies as policies
from zope.interface import implements
from twisted.cred.error import Unauthorized

from M2Crypto import SSL
from M2Crypto.SSL import TwistedProtocolWrapper
from M2Crypto.SSL.TwistedProtocolWrapper import TLSProtocolWrapper
from pyGridWare.utility.m2crypto import TLSProtocolSessionCachingWrapper as TLSProtocolWrapper

TwistedProtocolWrapper.debug = 1



###########################################################################
# Grab configuration information
#
###########################################################################
CP = ConfigParser()
CP.read('server-config.txt')
PORT = CP.getint('Location', 'port')
HOST = CP.get('Location', 'host')


def SetUpResourceTree(root):    
    core = Resource()
    root.putChild('wsrf', Resource())
    root.getStaticEntity('wsrf').putChild('core', core)
    wsrf = Resource()

    # Discover WS-Address URI
    addr = Address()
    addr.setUp()

    # Initialize singleton
    manager = SubscriptionManager.getInstance(SMServiceBinding,
                      {'wsAddressURI':addr.wsAddressURI,
                       'contextFactory':core.server.args[1]},)
    core.putChild('SubscriptionManagerService', manager)  
    
    
    #from generated.CounterService import Counter
    #counter = Counter()
    
    #guard = ResourcePortal(counter)
    #guard.isLeaf = 1
    #root.getStaticEntity('wsrf').putChild('services', wsrf)
    #wsrf.putChild('CounterService', guard)

    # Path "wsrf/services": Drop in directory "services"
    dropins = static.File("services")
    dropins.ignoreExt(".rpy")
    dropins.processors = {".rpy": script.ResourceScript}
    root.getStaticEntity('wsrf').putChild('services', dropins)
    
    return root


###########################################################################
# Set up Application
#
###########################################################################
class MyRequest(server.Request):

    def process(self):
        log.msg("Process a request.", debug=True)
        
        # get site from channel
        self.site = self.channel.site

        # set various default headers
        self.setHeader('server', server.version)
        self.setHeader('date', _http.datetimeToString())
        self.setHeader('content-type', "text/html")

        # Resource Identification
        self.prepath = []
        self.postpath = map(server.unquote, string.split(self.path[1:], '/'))
        try:
            resrc = self.site.getResourceFor(self)
            self.render(resrc)
        except:
            self.processingFailed(failure.Failure())
            
        self.method
        self.uri
        self.clientproto
        log.msg("CLIENT: %s" %self.client, debug=True)
        log.msg("HOST:   %s" %self.host, debug=True)

    
    
class MyHTTPChannel(HTTPChannel):
    
    def connectionMade(self):
        HTTPChannel.connectionMade(self)
        log.msg("CONNECTION MADE", debug=True)
        #avatar = self.site.portal.login((IAuthentication))

    def dataReceived(self, data):
        log.msg("DATA RECEIVED: %s" %self.requestFactory, debug=True)
        return HTTPChannel.dataReceived(self, data)


#class XPortal(Portal):
#    
#    def login(self, credentials, mind, *interfaces):
#        ifac = interface.providedBy(credentials)
#        log.msg("IFACS: %s" %ifac, debug=True)
#        for i in ifac:
#            c = self.checkers.get(i)
#            log.msg("IFAC: %s" %i, debug=True)
#            log.msg("CHECKER: %s" %c, debug=True)
#            if c is not None:
#                return maybeDeferred(c.requestAvatarId, credentials
#                    ).addCallback(self.realm.requestAvatar, mind, *interfaces
#                    )
#
#        log.msg("NOT FOUND", debug=True)
#        return defer.fail(failure.Failure(error.UnhandledCredentials(
#            "No checker for %s" % ', '.join(map(reflect.qual, ifac)))))


# SAMPLE CERTIFICATES
abs = lambda f: os.path.abspath(f)

key = abs(os.path.join(os.curdir, 'etc', 'certificates', 'server.key'))
cert = abs(os.path.join(os.curdir, 'etc', 'certificates', 'server.pem'))
calist = map(abs, [os.path.join(os.curdir, 'etc', 'certificates', 'ca.pem'), ])
capath = abs(os.path.join(os.curdir, 'etc', 'certificates'))


# SSL CONTEXT 
ctx = SSL.Context('sslv3')
class ContextFactory:
    def getContext(self): return ctx
    
ctx.set_cipher_list("NULL-MD5:ALL:!ADH:!EXP:@STRENGTH")
ctx.load_cert(cert, key)
ctx.set_allow_unknown_ca(False)
ctx.set_verify(SSL.verify_peer, 10)

for ca in calist: 
    log.msg('load CA: %s' %ca, debug=True)
    ctx.load_verify_locations(cafile=ca, capath=capath)


#class RootRealm(object):
#    implements(IRealm)
#    
#    def __init__(self, resource):
#        self.resource = resource
#
#    def requestAvatar(self, avatarId, mind, *interfaces):
#        """
#        avatarId -- subjectDN of peer certificate.
#        /C=US/ST=Oregon/L=Portland/O=LBNL/OU=CRD/CN=TestClient1
#        """
#        logout = lambda: None
#        
#        log.msg("AVATARID: %s" %avatarId, debug=True)
#        if IResource in interfaces:
#            if avatarId != '/C=US/ST=Oregon/L=Portland/O=LBNL/OU=CRD/CN=TestClient1':
#                 raise Unauthorized("%s" %avatarId)
#
#            return (IResource, self.resource, logout)
#            
#        raise NotImplementedError("Do not support any of the interfaces %s" %
#            (interfaces,))

_root = Resource()
#portal = XPortal(RootRealm(_root))
#portal.registerChecker(X509SubjectDNChecker())
#root = ResourcePortal(_root)

Site.protocol = MyHTTPChannel
Site.requestFactory = MyRequest
siteFactory = Site(_root)

wrapFactory = policies.WrappingFactory(siteFactory)
#wrapFactory.portal = portal
wrapFactory.protocol.TLS = True
wrapFactory.protocol = lambda factory, wrappedProtocol: \
    TLSProtocolWrapper(factory,
                       wrappedProtocol,
                       startPassThrough=0,
                       client=0,
                       contextFactory=ContextFactory(),
                       postConnectionCheck=None)


_root.__class__.server = port = internet.TCPServer(PORT, wrapFactory, 
                                                   interface=HOST)
SetUpResourceTree(_root)

application = service.Application("WSRF-SSL-Container")
port.setServiceParent(application)




